-- =============================================================================================================
-- *
-- * Copyright (c) University of York
-- *
-- * File Name: i2c_tx_port.vhd
-- *
-- * Version: V1.0
-- *
-- * Release Date:
-- *
-- * Author(s): M.Freeman
-- *
-- * Description: serial I2C tx port
-- *
-- * Change History:  $Author: $
-- *                  $Date: $
-- *                  $Revision: $
-- *
-- * Conditions of Use: THIS CODE IS COPYRIGHT AND IS SUPPLIED "AS IS" WITHOUT WARRANTY OF ANY KIND, INCLUDING,
-- *                    BUT NOT LIMITED TO, ANY IMPLIED WARRANTY OF MERCHANTABILITY AND FITNESS FOR A
-- *                    PARTICULAR PURPOSE.
-- *
-- * Notes:
-- *   
-- =============================================================================================================


LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;

ENTITY tx_port IS
PORT (
  clk:  IN STD_LOGIC;
  baud: IN STD_LOGIC;
  clr:  IN STD_LOGIC;
  wr:   IN STD_LOGIC;
  din:  IN STD_LOGIC_VECTOR (7 DOWNTO 0);
  tx : OUT STD_LOGIC;    
  idle: OUT STD_LOGIC );
END tx_port;

ARCHITECTURE tx_port_arch OF tx_port IS

  COMPONENT tx_bit_cnt
  PORT (
    clk: IN STD_LOGIC;
    baud: IN STD_LOGIC;
    clr: IN STD_LOGIC;
    rst: IN STD_LOGIC;
    en: IN STD_LOGIC;
    max: OUT STD_LOGIC );
  END COMPONENT;

  COMPONENT tx_shift_reg
  PORT (
    clk: IN STD_LOGIC;
    baud: IN STD_LOGIC;
    clr: IN STD_LOGIC;
    en: IN STD_LOGIC;
    ld: IN STD_LOGIC;
    data: IN STD_LOGIC_VECTOR(7 DOWNTO 0);
    tx: OUT STD_LOGIC);
  END COMPONENT;

  COMPONENT tx_sync
  PORT (
    clk: IN STD_LOGIC;
    baud: IN STD_LOGIC;
    clr: IN STD_LOGIC;
    rst: IN STD_LOGIC;
    sPnt: OUT STD_LOGIC );
  END COMPONENT;

  COMPONENT tx_state_mac
  PORT (
    clk: IN STD_LOGIC;
    baud: IN STD_LOGIC;
    clr: IN STD_LOGIC;
    sPnt: IN STD_LOGIC;
    bMax: IN STD_LOGIC;
    wr: IN STD_LOGIC;
    txRst: OUT STD_LOGIC;
    bCen: OUT STD_LOGIC;
    bRst: OUT STD_LOGIC;
    bSht: OUT STD_LOGIC;
    bLd: OUT STD_LOGIC;
    idle: OUT STD_LOGIC );
  END COMPONENT;

  COMPONENT tx_handshake
  PORT (
    clk: IN STD_LOGIC;
    clr: IN STD_LOGIC;
    wr_in: IN STD_LOGIC;
    wr_out: OUT STD_LOGIC;
    idle_in: IN STD_LOGIC;
    idle_out: OUT STD_LOGIC );
  END COMPONENT;

  COMPONENT reg IS
  GENERIC (
    width : INTEGER := 32 );
  PORT (
    clk : IN STD_LOGIC;
    clr : IN STD_LOGIC;
    en : IN STD_LOGIC;
    rst : IN STD_LOGIC;
    din : IN STD_LOGIC_VECTOR(width-1 DOWNTO 0);
    dout : OUT STD_LOGIC_VECTOR(width-1 DOWNTO 0));
  END COMPONENT;

  SIGNAL gnd: STD_LOGIC;
  SIGNAL bit_max: STD_LOGIC;
  SIGNAL tx_max: STD_LOGIC;
  SIGNAL bit_count_en: STD_LOGIC;
  SIGNAL bit_count_rst: STD_LOGIC;
  SIGNAL bit_shift_en: STD_LOGIC;
  SIGNAL bit_shift_ld: STD_LOGIC;
  SIGNAL tx_clk_rst: STD_LOGIC;
  SIGNAL dint: STD_LOGIC_VECTOR (7 DOWNTO 0);
  SIGNAL idle_int: STD_LOGIC;
  SIGNAL wr_int: STD_LOGIC;


BEGIN

  --
  -- signal buffers
  --

  gnd <= '0';
  
  --
  -- components
  --

  -- input data reg
  --
  tx_data_reg: reg
  GENERIC MAP(
    width => 8 )
  PORT MAP(
    clk => clk,
    clr => clr,
    en => wr,
    rst => gnd,
    din => din,
    dout => dint );

  -- bit counter
  --
  tx_bit_cnt_inst: tx_bit_cnt PORT MAP(
    clk => clk,
    baud =>  baud,
    clr => clr,
    rst => bit_count_rst,
    en => bit_count_en,
    max => bit_max );

  -- bit shift register
  --
  tx_shift_reg_inst: tx_shift_reg PORT MAP(
    clk => clk,
    baud => baud,
    clr => clr,
    en => bit_shift_en,
    ld => bit_shift_ld,
    data => dint,
    tx => tx );

  -- baud rate sync circuit
  --
  tx_sync_inst: tx_sync PORT MAP(
    clk => clk,
    baud => baud,
    clr => clr,
    rst => tx_clk_rst,
    sPnt => tx_max );

  -- controlling state machine
  --
  tx_state_mac_inst: tx_state_mac PORT MAP (
    clk => clk,
    baud => baud,
    clr => clr,
    sPnt => tx_max,
    bMax =>  bit_max,
    wr => wr_int,
    txRst => tx_clk_rst,
    bCen => bit_count_en,
    bRst =>  bit_count_rst,
    bSht => bit_shift_en,
    bLd => bit_shift_ld,
    idle => idle_int );

  -- handshake control
  --
  tx_handshake_inst : tx_handshake PORT MAP (
    clk => clk,
    clr => clr,
    wr_in => wr,
    wr_out => wr_int,
    idle_in => idle_int,
    idle_out => idle );


END tx_port_arch;
